/**
 * Wowza server software and all components Copyright 2006 - 2015, Wowza Media Systems, LLC, licensed pursuant to the Wowza Media Software End User License Agreement.
 */
package com.wowza.wms.plugin.collection.module;

import com.wowza.wms.amf.*;
import com.wowza.wms.application.*;
import com.wowza.wms.logging.*;
import com.wowza.wms.module.*;
import com.wowza.wms.stream.*;

public class ModuleFilterOnMetaData extends ModuleBase
{
    private static final String CLASSNAME = "ModuleFilterOnMetaData";
    private static final Class<ModuleFilterOnMetaData> CLASS = ModuleFilterOnMetaData.class;
    
    private String[] fieldsToRemove = new String[0];

    class StreamListener extends MediaStreamActionNotify3Base
	{
		public void onMetaData(IMediaStream stream, AMFPacket metaDataPacket)
		{
			WMSLoggerFactory.getLogger(CLASS).info(CLASSNAME+"#StreamListener.onMetaData["+stream.getContextStr()+"]: Filter on onMetaData event: size:"+metaDataPacket.getSize());

			try
			{
				while(true)
				{
					byte[] dataDataBytes = metaDataPacket.getData();
					if (dataDataBytes == null)
						break;

					if (metaDataPacket.getSize() <= 1)
						break;

					int dataStart = 0;
					if (dataDataBytes[0] == 0x00)
						dataStart++;

					AMFDataList dataList = new AMFDataList(dataDataBytes, dataStart, dataDataBytes.length-dataStart);
					
					if (dataList == null)
						break;

					int i = 0;
					for(;i<dataList.size();i++)
					{
						if (dataList.get(i).getType() != AMFData.DATA_TYPE_STRING)
							continue;
						
						String metaDataStr = dataList.getString(i);
						
						if (metaDataStr.equalsIgnoreCase("onMetaData"))
							break;
					}
					
					//System.out.println("********************* i:"+i+":"+(dataList.size()-1));
					
					boolean rewrite = false;
					String fieldsRemoved = "";
					if (i < dataList.size()-1)
					{
						i++;
						
						int dataType = dataList.get(i).getType();
						//System.out.println("********************* dataType:"+dataType);
						if (dataType == AMFData.DATA_TYPE_OBJECT || dataType == AMFData.DATA_TYPE_MIXED_ARRAY)
						{
							IAMFDataObj dataObj = (IAMFDataObj)dataList.get(i);
							//System.out.println("********************* dataObj.containsKey(gstfiller):"+dataObj.containsKey("gstfiller"));
							
							for(int j=0;j<fieldsToRemove.length;j++)
							{
								if (dataObj.containsKey(fieldsToRemove[j]))
								{
									dataObj.remove(fieldsToRemove[j]);
									if (fieldsRemoved.length() > 0)
										fieldsRemoved += ",";
									fieldsRemoved += fieldsToRemove[j];
									rewrite = true;
								}
							}
						}
					}

					if (rewrite)
					{
						byte[] metaDataData = dataList.serialize();
						if (metaDataData == null)
							break;
						
						WMSLoggerFactory.getLogger(CLASS).info(CLASSNAME+"#StreamListener.onMetaData["+stream.getContextStr()+"]: Rewriting onMetaData packet to remove ["+fieldsRemoved+"]: oldSize:"+metaDataPacket.getSize()+" newSize:"+metaDataData.length);

						metaDataPacket.setDataBuffer(metaDataData);
					}
					else
						WMSLoggerFactory.getLogger(CLASS).info(CLASSNAME+"#StreamListener.onMetaData["+stream.getContextStr()+"]: No need to filter");
					break;
				}
			}
			catch(Exception e)
			{
				WMSLoggerFactory.getLogger(CLASS).error(CLASSNAME+"#StreamListener.onMetaData["+stream.getContextStr()+"]: ", e);
			}
		}
	}
	
	public void onStreamCreate(IMediaStream stream)
	{
		if (fieldsToRemove.length > 0)
		{
			stream.addClientListener(new StreamListener());
		}
	}
	
	public void onAppStart(IApplicationInstance appInstance)
	{
		String fieldsToRemoveStr = null;
		fieldsToRemoveStr = appInstance.getProperties().getPropertyStr("filterOnMetaDataFieldsToRemove", fieldsToRemoveStr);
		if (fieldsToRemoveStr != null)
		{
			WMSLoggerFactory.getLogger(CLASS).info(CLASSNAME+".onAppStart["+appInstance.getContextStr()+"]: filterOnMetaDataFieldsToRemove: "+fieldsToRemoveStr);

			fieldsToRemoveStr = fieldsToRemoveStr.replace(",", "|");
			fieldsToRemove = fieldsToRemoveStr.split("[|]");
			for(int i=0;i<fieldsToRemove.length;i++)
			{
				fieldsToRemove[i] = fieldsToRemove[i].trim();
			}
		}
		else
			WMSLoggerFactory.getLogger(CLASS).warn(CLASSNAME+".onAppStart["+appInstance.getContextStr()+"]: filterOnMetaDataFieldsToRemove property is missing.");
	}
}
